/*
Copyright (C) Christopher Fogelberg, Mengjie Zhang 2005

This file is part of VUWLGP.

VUWLGP is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

VUWLGP is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with VUWLGP; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
  
#ifndef IFITNESSCASE_H
#define IFITNESSCASE_H

#include <vector>

namespace VUWLGP {
  // Represents a single fitness case (e.g. case to be classified, point in a symreg) with
  // features of type T.
  template <class T> class IFitnessCase {
  public:
    virtual ~IFitnessCase() { }

    // Returns the value of the i'th feature.
    T F(unsigned int i) const { return features[i]; }

    // Returns a new IFitnessCase of whichever class implements this function
    // The Config object currently is used, but who knows if it'll be needed in the future...
    // C++ doesn't allow abstract statics (for good reason, but it would be nice here if 
    // they could be virtual on the type) so every IFitnessCase should implement
    // a method that looks like this. The caller of this function is responsible for the memory.
    // static IFitnessCase<T>* Generate(const std::string& patternFileLine) = 0;

  protected:
    std::vector<T> features;

    // This method exists just to make this class an abstract class.
    virtual void MakeAbstract() const = 0;
  };
}
#endif
