/*
Copyright (C) Christopher Fogelberg, Mengjie Zhang 2005

This file is part of VUWLGP.

VUWLGP is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

VUWLGP is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with VUWLGP; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
  
#ifndef RAND_H
#define RAND_H

#include <cstdlib>
#include <ctime>

namespace VUWLGP {
  namespace Rand {
    // Initialise the random number generator with a time-dependent seed.
    void Init();
    
    // Initialise the random number generated with a specified seed.
    void Init(unsigned int seed);
    
    // Returns a random integer in the range [0, ceiling).
    unsigned int Int(unsigned int ceiling);
    
    // Returns a random integer in the range [0, RAND_MAX].
    unsigned int Int();
    
    // Returns a float in the range [0, 1).
    float Uniform();
    
    // Returns a float normally distributed from the mean
    double Normal(double mean,double sd);

    // Generates a random value of type T (e.g. for an ephemeral random constant)
    template <class T> T GenerateRandomConstant();
  }
}
#endif
